@@ -36,6 +36,10 @@ Net::Jabber::Client - Jabber Client Library
 
     $Con->Connect(hostname=>"jabber.org");
 
+    if ($Con->Connected()) {
+      print "We are connected to the server...\n";
+    }
+
     #
     # For the list of available function see Net::Jabber::Protocol.
     #
@@ -46,12 +50,14 @@ Net::Jabber::Client - Jabber Client Library
 
 =head2 Basic Functions
 
-    new(debug=>string)       - creates the Client object.  debug
-        debugfh=>FileHandle)   should be set to the path for the debug
-                               log to be written.  If set to "stdout" 
-                               then the debug will go there.  Also, you
-                               can specify a filehandle that already
-                               exists and use that.
+    new(debuglevel=>0|1|2, - creates the Client object.  debugfile
+        debugfile=>string)   should be set to the path for the debug
+                             log to be written.  If set to "stdout" 
+                             then the debug will go there.  debuglevel 
+                             controls the amount of debug.  For more
+                             information about the valid setting for
+                             debuglevel and debugfile see 
+                             Net::Jabber::Debug.
 
     Connect(hostname=>string, - opens a connection to the server listed
 	    port=>integer)      in the host name value, on the port
@@ -60,6 +66,9 @@ Net::Jabber::Client - Jabber Client Library
 
     Disconnect() - closes the connection to the server.
 
+    Connected() - returns 1 if the Transport is connected to the server,
+                  and 0 if not.
+
 =head1 AUTHOR
 
 By Ryan Eatmon in May of 2000 for http://jabber.org.
@@ -75,10 +84,9 @@ require 5.003;
 use strict;
 use XML::Stream;
 use IO::Select;
-use FileHandle;
 use vars qw($VERSION $AUTOLOAD);
 
-$VERSION = "1.0";
+$VERSION = "1.0005";
 
 use Net::Jabber::Protocol;
 ($Net::Jabber::Protocol::VERSION < $VERSION) &&
@@ -89,51 +97,26 @@ sub new {
   my $self = { };
 
   my %args;
-  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
+  while($#_ >= 0) { $args{ lc(pop(@_)) } = pop(@_); }
 
-  $self->{DEBUG} = 0;
-  if (exists($args{debugfh}) && ($args{debugfh} ne "")) {
-    $self->{DEBUGFILE} = $args{debugfh};
-    $self->{DEBUG} = 1;
-  }
-  if (((!exists($args{debugfh})) || 
-       (exists($args{debugfh}) && ($args{debugfh} eq ""))) && 
-      (exists($args{debug}) && ($args{debug} ne ""))) {
-    $self->{DEBUG} = 1;
-    if (lc($args{debug}) eq "stdout") {
-      $self->{DEBUGFILE} = new FileHandle(">&STDOUT");
-      $self->{DEBUGFILE}->autoflush(1);
-    } else {
-      if (-e $args{debug}) {
-	if (-w $args{debug}) {
-	  $self->{DEBUGFILE} = new FileHandle(">$args{debug}");
-	  $self->{DEBUGFILE}->autoflush(1);
-	} else {
-	  print "WARNING: debug file ($args{debug}) is not writable by you\n";
-	  print "         No debug information being saved.\n";
-	  $self->{DEBUG} = 0;
-	}
-      } else {
-	$self->{DEBUGFILE} = new FileHandle(">$args{debug}");
-	if (defined($self->{DEBUGFILE})) {
-	  $self->{DEBUGFILE}->autoflush(1);
-	} else {
-	  print "WARNING: debug file ($args{debug}) does not exist \n";
-	  print "         and is not writable by you.\n";
-	  print "         No debug information being saved.\n";
-	  $self->{DEBUG} = 0;
-	}
-      }
-    }
-  }
+  bless($self, $proto);
 
-  $self->{SERVER} = {"hostname" => "127.0.0.1", 
-		     "port" => 5222};
+  $self->{DELEGATE} = new Net::Jabber::Protocol();
 
-  $self->{STREAM} = new XML::Stream(debugfh=>$self->{DEBUGFILE})
-    if ($self->{DEBUG});
-  $self->{STREAM} = new XML::Stream()
-    if !($self->{DEBUG});
+  $self->{DEBUG} = 
+    new Net::Jabber::Debug(level=>exists($args{debuglevel}) ? $args{debuglevel} : -1,
+			   file=>exists($args{debugfile}) ? $args{debugfile} : "stdout",
+			   setdefault=>1,
+			   header=>"NJ::Client"
+			  );
+
+  $self->{SERVER} = {hostname => "localhost", 
+		     port => 5222};
+
+  $self->{CONNECTED} = 0;
+
+  $self->{STREAM} = new XML::Stream(debugfh=>$self->{DEBUG}->GetHandle(),
+				    debuglevel=>$self->{DEBUG}->GetLevel());
 
   $self->{VERSION} = $VERSION;
   
@@ -146,9 +129,6 @@ sub new {
     $self->{DIGEST} = 0;
   }
 
-  $self->{DELEGATE} = new Net::Jabber::Protocol();
-
-  bless($self, $proto);
   return $self;
 }
 
@@ -167,19 +147,6 @@ sub AUTOLOAD {
 }
 
 
-###########################################################################
-#
-# debug - prints the arguments to the debug log if debug is turned on.
-#
-###########################################################################
-sub debug {
-  my $self = shift;
-  return if ($self->{DEBUG} == 0);
-  my $fh = $self->{DEBUGFILE};
-  print $fh "Client: @_\n";
-}
-
-
 ###########################################################################
 #
 # Connect - Takes a has and opens the connection to the specified server.
@@ -195,14 +162,20 @@ sub Connect {
 
   while($#_ >= 0) { $self->{SERVER}{ lc pop(@_) } = pop(@_); }
 
-  $self->{SESSION} = 
+  $self->{DEBUG}->Log1("Connect: hostname($self->{SERVER}->{hostname})");
+
+  $self->{SESSION} =
     $self->{STREAM}->
       Connect(hostname=>$self->{SERVER}->{hostname},
 	      port=>$self->{SERVER}->{port},
 	      namespace=>"jabber:client"
-	     ) || return undef;
+	     ) || ($self->SetErrorCode($self->{STREAM}->GetErrorCode()) &&
+		   return);
   
+  $self->{DEBUG}->Log1("Connect: connection made");
+
   $self->{STREAM}->OnNode(sub{ $self->CallBack(@_) });
+  $self->{CONNECTED} = 1;
   return 1;
 }
 
@@ -215,7 +188,23 @@ sub Connect {
 sub Disconnect {
   my $self = shift;
 
-  $self->{STREAM}->Disconnect();
+  $self->{STREAM}->Disconnect() if ($self->{CONNECTED} == 1);
+  $self->{CONNECTED} = 0;
+  $self->{DEBUG}->Log1("Disconnect: bye bye");
+}
+
+
+###########################################################################
+#
+# Connected - returns 1 if the Transport is connected to the server, 0
+#             otherwise.
+#
+###########################################################################
+sub Connected {
+  my $self = shift;
+
+  $self->{DEBUG}->Log1("Connected: ($self->{CONNECTED})");
+  return $self->{CONNECTED};
 }
 
 
