@@ -274,7 +274,7 @@ use strict;
 use Carp;
 use vars qw($VERSION);
 
-$VERSION = "1.0";
+$VERSION = "1.0005";
 
 sub new {
   my $proto = shift;
@@ -286,12 +286,17 @@ sub new {
 
   bless($self, $proto);
 
+  $self->{DEBUG} = new Net::Jabber::Debug(usedefault=>1,
+                                          header=>"NJ::Message");
+
   if ("@_" ne ("")) {
     my @temp = @_;
     $self->{MESSAGE} = \@temp;
     my $xTree;
     foreach $xTree ($self->GetXTrees()) {
-      $self->AddX(@{$xTree});
+      my $xmlns = &Net::Jabber::GetXMLData("value",$xTree,"","xmlns");
+      next if !exists($Net::Jabber::DELEGATES{$xmlns});
+      $self->AddX($xmlns,@{$xTree});
     }
   } else {
     $self->{MESSAGE} = [ "message" , [{}]];
@@ -508,7 +513,7 @@ sub GetXTrees {
   my ($xmlns) = @_;
   my $xTree;
   my @xTrees;
-  foreach $xTree (&Net::Jabber::GetXMLData("tree array",$self->{MESSAGE},"x","xmlns",$xmlns)) {
+  foreach $xTree (&Net::Jabber::GetXMLData("tree array",$self->{MESSAGE},"*","xmlns",$xmlns)) {
     push(@xTrees,$xTree);
   }
   return @xTrees;
@@ -607,7 +612,7 @@ sub SetTo {
   my $self = shift;
   my ($to) = @_;
   if (ref($to) eq "Net::Jabber::JID") {
-    $to = $to->GetJID();
+    $to = $to->GetJID("full");
   }
   &Net::Jabber::SetXMLData("single",$self->{MESSAGE},"","",{to=>$to});
 }
@@ -622,7 +627,7 @@ sub SetFrom {
   my $self = shift;
   my ($from) = @_;
   if (ref($from) eq "Net::Jabber::JID") {
-    $from = $from->GetJID();
+    $from = $from->GetJID("full");
   }
   &Net::Jabber::SetXMLData("single",$self->{MESSAGE},"","",{from=>$from});
 }
@@ -746,7 +751,8 @@ sub SetError {
 sub NewX {
   my $self = shift;
   my ($xmlns) = @_;
-  my $xTag = $self->AddX();
+  return if !exists($Net::Jabber::DELEGATES{$xmlns});
+  my $xTag = $self->AddX($xmlns);
   $xTag->SetXMLNS($xmlns) if $xmlns ne "";
   return $xTag;
 }
@@ -761,8 +767,12 @@ sub NewX {
 ##############################################################################
 sub AddX {
   my $self = shift;
-  my (@xTree) = @_;
-  my $xTag = new Net::Jabber::X(@xTree);
+  my ($xmlns,@xTree) = @_;
+  return if !exists($Net::Jabber::DELEGATES{$xmlns});
+  $self->{DEBUG}->Log2("AddX: xmlns($xmlns) xTree(",\@xTree,")");
+  my $xTag;
+  eval("\$xTag = new ".$Net::Jabber::DELEGATES{$xmlns}->{parent}."(\@xTree);");
+  $self->{DEBUG}->Log2("AddX: xTag(",$xTag,")");
   push(@{$self->{XTAGS}},$xTag);
   return $xTag;
 }
@@ -782,28 +792,50 @@ sub AddX {
 sub MergeX {
   my $self = shift;
 
+  $self->{DEBUG}->Log2("MergeX: start");
+
   return if !(exists($self->{XTAGS}));
 
+  $self->{DEBUG}->Log2("MergeX: xTags(",$self->{XTAGS},")");
+
   my $xTag;
   my @xTags;
   foreach $xTag (@{$self->{XTAGS}}) {
     push(@xTags,$xTag);
   }
 
+  $self->{DEBUG}->Log2("MergeX: xTags(",\@xTags,")");
+  $self->{DEBUG}->Log2("MergeX: Check the old tags");
+  $self->{DEBUG}->Log2("MergeX: length(",$#{$self->{MESSAGE}->[1]},")");
+
+
   my $i;
   foreach $i (1..$#{$self->{MESSAGE}->[1]}) {
-    if ($self->{MESSAGE}->[1]->[$i] eq "x") {
+    $self->{DEBUG}->Log2("MergeX: i($i)");
+    $self->{DEBUG}->Log2("MergeX: data(",$self->{MESSAGE}->[1]->[$i],")");
+
+    if ((ref($self->{MESSAGE}->[1]->[($i+1)]) eq "ARRAY") &&
+	exists($self->{MESSAGE}->[1]->[($i+1)]->[0]->{xmlns})) {
+      $self->{DEBUG}->Log2("MergeX: found a namespace xmlns(",$self->{MESSAGE}->[1]->[($i+1)]->[0]->{xmlns},")");
+      next if !exists($Net::Jabber::DELEGATES{$self->{MESSAGE}->[1]->[($i+1)]->[0]->{xmlns}});
+      $self->{DEBUG}->Log2("MergeX: merge index($i)");
       my $xTag = pop(@xTags);
+      $self->{DEBUG}->Log2("MergeX: merge xTag($xTag)");
       my @xTree = $xTag->GetTree();
+      $self->{DEBUG}->Log2("MergeX: merge xTree(",\@xTree,")");
       $self->{MESSAGE}->[1]->[($i+1)] = $xTree[1];
     }
   }
 
+  $self->{DEBUG}->Log2("MergeX: Insert new tags");
   foreach $xTag (@xTags) {
+    $self->{DEBUG}->Log2("MergeX: new tag");
     my @xTree = $xTag->GetTree();
     $self->{MESSAGE}->[1]->[($#{$self->{MESSAGE}->[1]}+1)] = "x";
     $self->{MESSAGE}->[1]->[($#{$self->{MESSAGE}->[1]}+1)] = $xTree[1];
   }
+
+  $self->{DEBUG}->Log2("MergeX: end");
 }
 
 
@@ -820,6 +852,9 @@ sub Reply {
 
   my $reply = new Net::Jabber::Message();
 
+  my $subject = $self->GetSubject();
+  $subject =~ s/re\:\s+//i;
+  $reply->SetSubject("re: $subject");
   $reply->SetThread($self->GetThread()) if ($self->GetThread() ne "");
   $reply->SetID($self->GetID()) if ($self->GetID() ne "");
   $reply->SetType($self->GetType()) if ($self->GetType() ne "");
