@@ -35,19 +35,22 @@ Net::Jabber::Protocol - Jabber Protocol Library
 
     use Net::Jabber;
 
-    $Con = new Net::Jabber::Client();    # From Net::Jabber::Client
-    $Con->Connect(name=>"jabber.org");   #
+    $Con = new Net::Jabber::Client();            # From Net::Jabber::Client
+    $status = $Con->Connect(name=>"jabber.org"); #
 
       or
 
-    $Con = new Net::Jabber::Transport(); #
-    $Con->Connect(name=>"jabber.org",    # From Net::Jabber::Transport
-		  secret=>"bob");        #
+    $Con = new Net::Jabber::Transport();         #
+    $status = $Con->Connect(name=>"jabber.org",  # From Net::Jabber::Transport
+			    secret=>"bob");      #
 
 
     $Con->SetCallBacks(message=>\&messageCallBack,
 		       iq=>\&handleTheIQTag);
 
+    $error = $Con->GetErrorCode();
+    $Con->SetErrorCode("Timeout limit reached");
+
     $Con->Process();
     $Con->Process(5);
 
@@ -66,6 +69,15 @@ Net::Jabber::Protocol - Jabber Protocol Library
     $receiveObj = $Con->GetID($id);    
     $receiveObj = $Con->WaitForID($id);
 
+=head2 Namespace Functions
+
+    $Con->AddDelegate(namespace=>"foo::bar",
+                      parent=>"Foo::Bar");
+
+    $Con->AddDelegate(namespace=>"foo::bar::bob",
+                      parent=>"Foo::Bar",
+                      delegate=>"Foo::Bar::Bob");
+
 =head2 Message Functions
 
     $Con->MessageSend(to=>"bob@jabber.org",
@@ -81,20 +93,23 @@ Net::Jabber::Protocol - Jabber Protocol Library
 =head2 Subscription Functions
 
     $Con->Subscription(type=>"subscribe",
-		       jid=>"bob@jabber.org");
+		       to=>"bob@jabber.org");
 
     $Con->Subscription(type=>"unsubscribe",
-		       jid=>"bob@jabber.org");
+		       to=>"bob@jabber.org");
 
     $Con->Subscription(type=>"subscribed",
-		       jid=>"bob@jabber.org");
+		       to=>"bob@jabber.org");
 
     $Con->Subscription(type=>"unsubscribed",
-		       jid=>"bob@jabber.org");
+		       to=>"bob@jabber.org");
 
 =head2 IQ  Functions
 
-    $Con->SetQueryDelegates("com:bar:foo"=>"Foo::Bar");
+=head2 IQ::Agents Functions
+
+    %agents = $Con->AgentsGet();
+    %agents = $Con->AgentsGet(to=>"transport.jabber.org");
 
 =head2 IQ::Auth Functions
 
@@ -113,9 +128,14 @@ Net::Jabber::Protocol - Jabber Protocol Library
 
 =head2 IQ::Register Functions
 
+    %fields = $Con->RegisterRequest();
+    %fields = $Con->RegisterRequest(to=>"transport.jabber.org");
+
     @result = $Con->RegisterSend(usersname=>"newuser",
 				 resource=>"New User",
-				 password=>"imanewbie");
+				 password=>"imanewbie",
+                                 email=>"newguy@new.com",
+                                 key=>"some key");
 
 =head2 IQ::Resource Functions
 
@@ -123,11 +143,25 @@ Net::Jabber::Protocol - Jabber Protocol Library
 
 =head2 IQ::Roster Functions
 
+    %roster = $Con->RosterParse($iq);
     %roster = $Con->RosterGet();
-    $Con->RosterAdd(jid=>"bob@jabber.org");
+    $Con->RosterAdd(jid=>"bob\@jabber.org",
+		    name=>"Bob");
     $Con->RosterRemove(jid=>"bob@jabber.org");
 
 
+=head2 IQ::Search Functions
+
+    %fields = $Con->SearchRequest();
+    %fields = $Con->SearchRequest(to=>"users.jabber.org");
+
+    $Con->SearchSend(name=>"",
+                     first=>"Bob",
+                     last=>"",
+                     nick=>"bob",
+                     email=>"",
+                     key=>"som key");
+
 =head2 IQ::Time Functions
 
     %result = $Con->TimeQuery();
@@ -147,12 +181,17 @@ Net::Jabber::Protocol - Jabber Protocol Library
 
 =head2 X Functions
 
-    $Con->SetXDelegates("com:bar:foo"=>"Foo::Bar");
-
 =head1 METHODS
 
 =head2 Basic Functions
 
+    GetErrorCode() - returns a string that will hopefully contain some
+                     useful information about why a function returned
+                     an undef to you.
+
+    SetErrorCode(string) - set a useful error message before you return
+                           an undef to the caller.
+
     SetCallBacks(message=>function,  - sets the callback functions for
                  presence=>function,   the top level tags listed.  The
 		 iq=>function)         available tags to look for are
@@ -212,6 +251,21 @@ Net::Jabber::Protocol - Jabber Protocol Library
            <presence/> with an id is a risk.  Both clients must support 
            this for these functions to work.
 
+=head2 Namespace Functions
+
+    AddNamespace(namespace=>string, - this tells the Net::Jabber modules
+                 parent=>string,      about the new namespace.  The
+                 delegate=>string)    namespaces determines how the xmlns
+                                      looks in the tag.  The parent is
+                                      the name of the module to create
+                                      when you use this namespace.  The
+                                      delegate is only needed if the
+                                      parent module uses delegates to
+                                      distinguish between namespaces
+                                      (like the Net::Jabber::IQ and
+                                      Net::Jabber::X modules do).  The
+                                      delegate must be a valid Perl
+                                      Module.
 
 =head2 Message Functions
 
@@ -227,22 +281,46 @@ Net::Jabber::Protocol - Jabber Protocol Library
 
 =head2 Subscription Functions
 
-    Subscription(type=>string, - sends a Presence to the server to tell it
-                 jid=>string)    to perform the subscription in type to the
-                                 you to the specified JID.  The allowed
-                                 types are:
+    Subscription(hash) - taks the hash and passes it to SetPresence in
+                         Net::Jabber::Presence (refer there for valid
+                         settings).  Then it sends the subscription to
+                         server.
+
+                         The valid types of subscription are:
+
+                           subscribe    - subscribe to JID's presence
+                           unsubscribe  - unsubscribe from JID's presence
+                           subscribed   - response to a subscribe
+                           unsubscribed - response to an unsubscribe
 
-                                 subscribe    - subscribe to JID's presence
-                                 unsubscribe  - unsubscribe from JID's presence
-                                 subscribed   - response to a subscribe
-                                 unsubscribed - response to an unsubscribe
-                                 
 =head2 IQ Functions
 
-    SetQueryDelegates(hash) - the hash gets sent to the 
-                              Net::Jabber::Query::SetDelegates function.
-                              For more information about this function,
-                              read the manpage for Net::Jabber::Query.
+=head2 IQ::Agents Functions
+
+    AgentsGet(to=>string, - takes all of the information and
+    AgentsGet()             builds a Net::Jabber::IQ::Agents packet.
+                            It then sends that packet either to the
+                            server, or to the specified transport,
+                            with an ID and waits for that ID to return.
+                            Then it looks in the resulting packet and 
+                            builds a hash that contains the values
+                            of the agent list.  The hash is layed out
+                            like this:  (NOTE: the jid is the key to
+                            distinguish the various agents)
+
+                              $hash{<JID>}->{order} = 4
+                                          ->{name} = "ICQ Transport"
+                                          ->{transport} = "ICQ #"
+                                          ->{description} = "ICQ...blah..."
+                                          ->{service} = "icq"
+                                          ->{register} = 1
+                                          ->{search} = 1
+                                        etc...
+
+                            The order field determines the order that
+                            it came from the server in... in case you
+                            care.  For more info on the valid fields 
+                            see the Net::Jabber::Query::Agent module.
 
 =head2 IQ::Auth Functions
 
@@ -273,21 +351,31 @@ Net::Jabber::Protocol - Jabber Protocol Library
 
 =head2 IQ::Register Functions
 
-    RegisterSend(username=>string, - takes all of the information and
-                 password=>string,   builds a Net::Jabber::IQ::Register
-                 resource=>string)   packet.  It then sends that packet
-                                     to the server with an ID and waits
-                                     for that ID to return.  Then it
-                                     looks in resulting packet and
-                                     determines if registration was
-                                     successful for not.  The array
-                                     returned from RegisterSend looks
-                                     like this:
-                                       [ type , message ]
-                                     If type is "ok" then registration
-                                     was successful, otherwise message
-                                     contains a little more detail about the
-                                     error.
+    RegisterRequest(to=>string) - send an <iq/> request to the specified
+    RegisterRequest()             server/transport, if not specified it
+                                  sends to the current active server.
+                                  The function returns a hash that
+                                  contains the required fields.   Here
+                                  is an example of the hash:
+
+	                             $fields{intructions} = "do this..."
+                                     $fields{key} = "some key"
+                                     $fields{username} = ""
+                                     ...
+
+                                  The fields that are present are the
+                                  required fields the server needs.
+
+    RegisterSend(hash) - takes the contents of the hash and passes it
+	                 to the SetRegister function in the module
+                         Net::Jabber::Query::Register.  This function
+	                 returns an array that looks like this:
+  
+                            [ type , message ]
+
+                         If type is "ok" then registration was 
+                         successful, otherwise message contains a 
+                         little more detail about the error.
 
 =head2 IQ::Resource Functions
 
@@ -295,11 +383,8 @@ Net::Jabber::Protocol - Jabber Protocol Library
 
 =head2 IQ::Roster Functions
 
-    RosterGet() - sends an empty Net::Jabber::IQ::Roster tag to the
-                  server so the server will send the Roster to the
-                  client.  Then it takes the result and puts it into
-                  the following data structure which it returns to
-                  to the caller:
+    RosterParse(IQ object) - returns a hash that contains the roster
+                             parsed into the following data structure:
 
                   $roster{'bob@jabber.org'}->{name}         
                                       - Name you stored in the roster
@@ -315,12 +400,46 @@ Net::Jabber::Protocol - Jabber Protocol Library
 		  $roster{'bob@jabber.org'}->{groups}
                                       - Array of groups that 
                                         bob@jabber.org is in
-			  
-    RosterAdd(jid=>string) - sends a packet asking that the jid be
-                             added to the user's roster.
 
-    RosterRemove(jid=>string) - sends a packet asking that the jid be
-                             removed from the user's roster.
+    RosterGet() - sends an empty Net::Jabber::IQ::Roster tag to the
+                  server so the server will send the Roster to the
+                  client.  Returns the above hash from RosterParse.
+			  
+    RosterAdd(hash) - sends a packet asking that the jid be
+                      added to the roster.  The hash format
+	              is defined in the SetItem function
+                      in the Net::Jabber::Query::Roster::Item
+                      module.
+
+    RosterRemove(hash) - sends a packet asking that the jid be
+                         removed from the roster.  The hash
+	                 format is defined in the SetItem function
+                         in the Net::Jabber::Query::Roster::Item
+                         module.
+
+=head2 IQ::Search Functions
+
+    SearchRequest(to=>string) - send an <iq/> request to the specified
+    SearchRequest()             server/transport, if not specified it
+                                sends to the current active server.
+                                The function returns a hash that
+                                contains the required fields.   Here
+                                is an example of the hash:
+
+	                           $fields{intructions} = "do this..."
+                                   $fields{key} = "some key"
+                                   $fields{name} = ""
+                                   ...
+
+                                The fields that are present are the
+                                required fields the server needs.  If
+                                the hash is undefined then there was
+                                an error with the request.
+
+    SearchSend(to=>string|JID, - takes the contents of the hash and 
+	       hash)             passes it to the SetSearch function
+                                 in the module Net::Jabber::Query::Search.
+                                 And then send the packet.
 
 =head2 IQ::Time Functions
 
@@ -354,11 +473,6 @@ Net::Jabber::Protocol - Jabber Protocol Library
 
 =head2 X Functions
 
-    SetXDelegates(hash) - the hash gets sent to the 
-                          Net::Jabber::X::SetDelegates function.  For 
-                          more information about this function, read 
-                          the manpage for Net::Jabber::X.
-
 =head1 AUTHOR
 
 Revised by Ryan Eatmon in December 1999.
@@ -378,7 +492,7 @@ it under the same terms as Perl itself.
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "1.0";
+$VERSION = "1.0005";
 
 sub new {
   my $proto = shift;
@@ -391,6 +505,33 @@ sub new {
 }
 
 
+##############################################################################
+#
+# GetErrorCode - if you are returned an undef, you can call this function
+#                and hopefully learn more information about the problem.
+#
+##############################################################################
+sub GetErrorCode {
+  shift;
+  my $self = shift;
+  return ($self->{ERRORCODE} ne "") ? $self->{ERRORCODE} : $!;
+}
+
+
+##############################################################################
+#
+# SetErrorCode - sets the error code so that the caller can find out more
+#                information about the problem
+#
+##############################################################################
+sub SetErrorCode {
+  shift;
+  my $self = shift;
+  my ($errorcode) = @_;
+  $self->{ERRORCODE} = $errorcode;
+}
+
+
 ###########################################################################
 #
 # CallBack - Central callback function.  If a packet comes back with an ID
@@ -408,17 +549,17 @@ sub CallBack {
   my $self = shift;
   my (@object) = @_;
 
-  $self->debug("CallBack: received(",&Net::Jabber::BuildXML(@object),")");
+  $self->{DEBUG}->Log1("CallBack: received(",&Net::Jabber::BuildXML(@object),")");
 
   my $tag = $object[0];
   my $id;
   $id = $object[1]->[0]->{id} if (exists($object[1]->[0]->{id}));
 
-  $self->debug("CallBack: tag($tag)");
-  $self->debug("CallBack: id($id)") if ($id ne "");
+  $self->{DEBUG}->Log1("CallBack: tag($tag)");
+  $self->{DEBUG}->Log1("CallBack: id($id)") if ($id ne "");
 
   if ($self->CheckID($tag,$id)) {
-    $self->debug("CallBack: found registry entry: tag($tag) id($id)");
+    $self->{DEBUG}->Log1("CallBack: found registry entry: tag($tag) id($id)");
     $self->DeregisterID($tag,$id);
     my $NJObject;
     $NJObject = new Net::Jabber::IQ(@object) 
@@ -429,12 +570,12 @@ sub CallBack {
       if ($tag eq "message");
     $self->GotID($object[1]->[0]->{id},$NJObject);
   } else {
-    $self->debug("CallBack: no registry entry");  
+    $self->{DEBUG}->Log1("CallBack: no registry entry");  
     if (exists($self->{CB}->{$tag})) {
-      $self->debug("CallBack: goto user function($self->{CB}->{$tag})");
+      $self->{DEBUG}->Log1("CallBack: goto user function($self->{CB}->{$tag})");
       &{$self->{CB}->{$tag}}(@object);
     } else {
-      $self->debug("CallBack: no defined function.  Dropping packet.");
+      $self->{DEBUG}->Log1("CallBack: no defined function.  Dropping packet.");
     }
   }
 }
@@ -454,7 +595,7 @@ sub SetCallBacks {
   while($#_ >= 0) {
     my $func = pop(@_);
     my $tag = pop(@_);
-    $self->debug("SetCallBacks: tag($tag) func($func)");
+    $self->{DEBUG}->Log1("SetCallBacks: tag($tag) func($func)");
     $self->{CB}{$tag} = $func;
   }
 }
@@ -476,7 +617,7 @@ sub Process {
   my ($timeout) = @_;
   my ($status);
 
-  $self->debug("Process: timeout($timeout)");
+  $self->{DEBUG}->Log1("Process: timeout($timeout)");
 
   if (!defined($timeout) || ($timeout eq "")) {
     while(1) {
@@ -484,7 +625,7 @@ sub Process {
       last if (($status != 0) || ($status eq ""));
       select(undef,undef,undef,.25);
     }
-    $self->debug("Process: return($status)");
+    $self->{DEBUG}->Log1("Process: return($status)");
     return $status;
   } else {
     return $self->{STREAM}->Process($timeout);
@@ -520,7 +661,7 @@ sub SendXML {
   shift;
   my $self = shift;
   my($xml) = @_;
-  $self->debug("SendXML: sent($xml)");
+  $self->{DEBUG}->Log1("SendXML: sent($xml)");
   $self->{STREAM}->Send($xml);
 }
 
@@ -696,6 +837,23 @@ sub DeregisterID {
 }
 
 
+##############################################################################
+#
+# AddDelegate - adds the namespace and corresponding pacakge onto the list
+#               of availbale delegates based on the namespace.
+#
+##############################################################################
+sub AddDelegate {
+  my $self = shift;
+  my %delegates;
+  while($#_ >= 0) { $delegates{ lc pop(@_) } = pop(@_); }
+
+  $Net::Jabber::DELEGATES{$delegates{namespace}}->{parent} = $delegates{parent};
+  $Net::Jabber::DELEGATES{$delegates{namespace}}->{delegate} = $delegates{delegate};
+}
+
+
+
 ###########################################################################
 #
 # MessageSend - Takes the same hash that Net::Jabber::Message->SetMessage
@@ -721,6 +879,7 @@ sub PresenceSend {
   shift;
   my $self = shift;
   my $presence = new Net::Jabber::Presence();
+  $presence->SetPresence(@_);
   $self->Send($presence);
 }
 
@@ -733,9 +892,13 @@ sub PresenceSend {
 sub PresenceProbe {
   shift;
   my $self = shift;
+  my %args;
+  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
+  delete($args{type});
 
   my $presence = new Net::Jabber::Presence();
-  $presence->SetPresence(type=>"probe");
+  $presence->SetPresence(type=>"probe",
+			 %args);
   $self->Send($presence);
 }
 
@@ -749,38 +912,51 @@ sub PresenceProbe {
 sub Subscription {
   shift;
   my $self = shift;
-  my %args;
-  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
-  
-  if (($args{type} ne "subscribe") && ($args{type} ne "unsubscribe") &&
-      ($args{type} ne "subscribed") && ($args{type} ne "unsubscribed")) {
-    print STDERR "ERROR: Subscription must take one of the following types:\n";
-    print STDERR "         subscribe, unsubscribe, subscribed, unsubscribed\n";
-    exit(0);
-  }
-  if ($args{jid} eq "") {
-    print STDERR "ERROR: You must specify a Jabber ID for the Subscription\n";
-    exit(0);
-  }
 
   my $presence = new Net::Jabber::Presence();
-  $presence->SetPresence(type=>$args{type},
-			 to=>$args{jid});
+  $presence->SetPresence(@_);
   $self->Send($presence);
 }
 
 
 ###########################################################################
 #
-# SetQueryDelegates - Sets the delegates for the <query/> that you might
-#                     see during the session.
+# AgentsGet - Sends an empty IQ to the server/transport to request that the
+#             list of supported Agents be sent to them.  Returns a hash
+#             containing the values for the agents.
 #
 ###########################################################################
-sub SetQueryDelegates {
+sub AgentsGet {
   shift;
   my $self = shift;
-  my $query = new Net::Jabber::Query;
-  $query->SetDelegates(@_);
+
+  my $iq = new Net::Jabber::IQ;
+  $iq->SetIQ(@_);
+  $iq->SetIQ(type=>"get");
+  my $query = $iq->NewQuery("jabber:iq:agents");
+
+  $iq = $self->SendAndReceiveWithID($iq);
+
+  $query = $iq->GetQuery();
+  my @agents = $query->GetAgents();
+
+  my %agents;
+  my $agent;
+  my $count = 0;
+  foreach $agent (@agents) {
+    my $jid = $agent->GetJID();
+    $agents{$jid}->{name} = $agent->GetName();
+    $agents{$jid}->{description} = $agent->GetDescription();
+    $agents{$jid}->{transport} = $agent->GetTransport();
+    $agents{$jid}->{service} = $agent->GetService();
+    $agents{$jid}->{register} = $agent->GetRegister();
+    $agents{$jid}->{search} = $agent->GetSearch();
+    $agents{$jid}->{groupchat} = $agent->GetGroupChat();
+    $agents{$jid}->{agents} = $agent->GetAgents();
+    $agents{$jid}->{order} = $count++;
+  }
+
+  return %agents;
 }
 
 
@@ -794,7 +970,6 @@ sub SetQueryDelegates {
 sub AuthSend {
   shift;
   my $self = shift;
-
   my %args;
   while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
 
@@ -817,7 +992,7 @@ sub AuthSend {
   #------------------------------------------------------------------------
   # Create a Net::Jabber::IQ object to send to the server
   #------------------------------------------------------------------------
-  my $IQLogin = new Net::Jabber::IQ;
+  my $IQLogin = new Net::Jabber::IQ();
   my $IQAuth = $IQLogin->NewQuery("jabber:iq:auth");
   $IQAuth->SetAuth(%args);
 
@@ -837,6 +1012,52 @@ sub AuthSend {
 }
 
 
+###########################################################################
+#
+# RegisterRequest - This is a self contained function to send an iq tag
+#                   an id that requests the target address to send back
+#                   the required fields.  It waits for a reply what the
+#                   same id to come back and tell the caller what the 
+#                   fields are.
+#
+###########################################################################
+sub RegisterRequest {
+  shift;
+  my $self = shift;
+  my %args;
+  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
+
+  #------------------------------------------------------------------------
+  # Create a Net::Jabber::IQ object to send to the server
+  #------------------------------------------------------------------------
+  my $IQ = new Net::Jabber::IQ();
+  $IQ->SetIQ(to=>delete($args{to})) if exists($args{to});
+  $IQ->SetIQ(type=>"get");
+  my $IQRegister = $IQ->NewQuery("jabber:iq:register");
+
+  #------------------------------------------------------------------------
+  # Send the IQ with the next available ID and wait for a reply with that 
+  # id to be received.  Then grab the IQ reply.
+  #------------------------------------------------------------------------
+  $IQ = $self->SendAndReceiveWithID($IQ);
+  
+  #------------------------------------------------------------------------
+  # Check if there was an error.
+  #------------------------------------------------------------------------
+  if ($IQ->GetType() eq "error") {
+    $self->SetErrorCode($IQ->GetErrorCode().": ".$IQ->GetError());
+    return;
+  }
+
+  #------------------------------------------------------------------------
+  # From the reply IQ determine what fields are required and send a hash
+  # back with the fields and any values that are already defined (like key)
+  #------------------------------------------------------------------------
+  $IQRegister = $IQ->GetQuery();
+  return %{$IQRegister->GetFields()};
+}
+
+
 ###########################################################################
 #
 # RegisterSend - This is a self contained function to send a registration
@@ -847,14 +1068,17 @@ sub AuthSend {
 sub RegisterSend {
   shift;
   my $self = shift;
+  my %args;
+  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
 
   #------------------------------------------------------------------------
   # Create a Net::Jabber::IQ object to send to the server
   #------------------------------------------------------------------------
-  my $IQ = new Net::Jabber::IQ;
+  my $IQ = new Net::Jabber::IQ();
+  $IQ->SetIQ(to=>delete($args{to})) if exists($args{to});
   $IQ->SetIQ(type=>"set");
   my $IQRegister = $IQ->NewQuery("jabber:iq:register");
-  $IQRegister->SetRegister(@_);
+  $IQRegister->SetRegister(%args);
 
   #------------------------------------------------------------------------
   # Send the IQ with the next available ID and wait for a reply with that 
@@ -881,12 +1105,16 @@ sub RegisterSend {
 sub RosterAdd {
   shift;
   my $self = shift;
+  my %args;
+  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
 
-  my $iq = new Net::Jabber::IQ;
+  my $iq = new Net::Jabber::IQ();
+  $iq->SetIQ(type=>"set");
   my $roster = $iq->NewQuery("jabber:iq:roster");
   my $item = $roster->AddItem();
-  $item->SetItem(@_,
-		 subscription=>"to");
+  $item->SetItem(%args);
+
+  $self->{DEBUG}->Log1("RosterAdd: xml(",$iq->GetXML(),")");
   $self->Send($iq);
 }
 
@@ -900,11 +1128,15 @@ sub RosterAdd {
 sub RosterRemove {
   shift;
   my $self = shift;
+  my %args;
+  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
+  delete($args{subscription});
 
-  my $iq = new Net::Jabber::IQ;
+  my $iq = new Net::Jabber::IQ();
+  $iq->SetIQ(type=>"set");
   my $roster = $iq->NewQuery("jabber:iq:roster");
   my $item = $roster->AddItem();
-  $item->SetItem(@_,
+  $item->SetItem(%args,
 		 subscription=>"remove");
   $self->Send($iq);
 }
@@ -912,21 +1144,15 @@ sub RosterRemove {
 
 ###########################################################################
 #
-# RosterGet - Sends an empty IQ to the server to request that the user's
-#             Roster be sent to them.
+# RosterParse - Returns a hash of roster items.
 #
 ###########################################################################
-sub RosterGet {
+sub RosterParse {
   shift;
   my $self = shift;
+  my($iq) = @_;
 
-  my $iq = new Net::Jabber::IQ;
-  $iq->SetIQ(type=>"get");
-  my $query = $iq->NewQuery("jabber:iq:roster");
-
-  $iq = $self->SendAndReceiveWithID($iq);
-
-  $query = $iq->GetQuery();
+  my $query = $iq->GetQuery();
   my @items = $query->GetItems();
 
   my %roster;
@@ -943,6 +1169,108 @@ sub RosterGet {
 }
 
 
+###########################################################################
+#
+# RosterGet - Sends an empty IQ to the server to request that the user's
+#             Roster be sent to them.  Returns a hash of roster items.
+#
+###########################################################################
+sub RosterGet {
+  shift;
+  my $self = shift;
+
+  my $iq = new Net::Jabber::IQ;
+  $iq->SetIQ(type=>"get");
+  my $query = $iq->NewQuery("jabber:iq:roster");
+
+  $iq = $self->SendAndReceiveWithID($iq);
+
+  return $self->RosterParse($iq);
+}
+
+
+###########################################################################
+#
+# SearchRequest - This is a self contained function to send an iq tag
+#                 an id that requests the target address to send back
+#                 the required fields.  It waits for a reply what the
+#                 same id to come back and tell the caller what the 
+#                 fields are.
+#
+###########################################################################
+sub SearchRequest {
+  shift;
+  my $self = shift;
+  my %args;
+  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
+
+  #------------------------------------------------------------------------
+  # Create a Net::Jabber::IQ object to send to the server
+  #------------------------------------------------------------------------
+  my $IQ = new Net::Jabber::IQ();
+  $IQ->SetIQ(to=>delete($args{to})) if exists($args{to});
+  $IQ->SetIQ(type=>"get");
+  my $IQSearch = $IQ->NewQuery("jabber:iq:search");
+
+  $self->{DEBUG}->Log1("SearchRequest: sent(",$IQ->GetXML(),")");
+
+  #------------------------------------------------------------------------
+  # Send the IQ with the next available ID and wait for a reply with that 
+  # id to be received.  Then grab the IQ reply.
+  #------------------------------------------------------------------------
+  $IQ = $self->SendAndReceiveWithID($IQ);
+  
+  $self->{DEBUG}->Log1("SearchRequest: received(",$IQ->GetXML(),")");
+
+  #------------------------------------------------------------------------
+  # Check if there was an error.
+  #------------------------------------------------------------------------
+  if ($IQ->GetType() eq "error") {
+    $self->SetErrorCode($IQ->GetErrorCode().": ".$IQ->GetError());
+    $self->{DEBUG}->Log1("SearchRequest: error(",$self->GetErrorCode(),")");
+    return;
+  }
+
+  #------------------------------------------------------------------------
+  # From the reply IQ determine what fields are required and send a hash
+  # back with the fields and any values that are already defined (like key)
+  #------------------------------------------------------------------------
+  $IQSearch = $IQ->GetQuery();
+  $self->{DEBUG}->Log1("SearchRequest: return(",\%{$IQSearch->GetFields()},")");
+  return %{$IQSearch->GetFields()};
+}
+
+
+###########################################################################
+#
+# SearchSend - This is a self contained function to send a search
+#              iq tag with an id.  Then wait for a reply what the same
+#              id to come back and tell the caller what the result was.
+#
+###########################################################################
+sub SearchSend {
+  shift;
+  my $self = shift;
+  my %args;
+  while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
+
+  #------------------------------------------------------------------------
+  # Create a Net::Jabber::IQ object to send to the server
+  #------------------------------------------------------------------------
+  my $IQ = new Net::Jabber::IQ();
+  $IQ->SetIQ(to=>delete($args{to})) if exists($args{to});
+  $IQ->SetIQ(type=>"set");
+  my $IQSearch = $IQ->NewQuery("jabber:iq:search");
+  $IQSearch->SetSearch(%args);
+
+  #------------------------------------------------------------------------
+  # Send the IQ with the next available ID and wait for a reply with that 
+  # id to be received.  Then grab the IQ reply.
+  #------------------------------------------------------------------------
+  $self->Send($IQ);
+}
+
+
 ###########################################################################
 #
 # TimeQuery - Sends an iq:time query to either the server or the specified
@@ -956,7 +1284,7 @@ sub TimeQuery {
   while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
 
   my $iq = new Net::Jabber::IQ();
-  $iq->SetIQ(to=>$args{to}) if exists($args{to});
+  $iq->SetIQ(to=>delete($args{to})) if exists($args{to});
   $iq->SetIQ(type=>'get');
   my $time = $iq->NewQuery("jabber:iq:time");
 
@@ -976,10 +1304,10 @@ sub TimeSend {
   while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
 
   my $iq = new Net::Jabber::IQ();
-  $iq->SetIQ(to=>$args{to},
+  $iq->SetIQ(to=>delete($args{to}),
 	     type=>'result');
   my $time = $iq->NewQuery("jabber:iq:time");
-  $time->SetTime();
+  $time->SetTime(%args);
 
   $self->Send($iq);
 }
@@ -999,7 +1327,7 @@ sub VersionQuery {
   while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
 
   my $iq = new Net::Jabber::IQ();
-  $iq->SetIQ(to=>$args{to}) if exists($args{to});
+  $iq->SetIQ(to=>delete($args{to})) if exists($args{to});
   $iq->SetIQ(type=>'get');
   my $version = $iq->NewQuery("jabber:iq:version");
 
@@ -1028,18 +1356,4 @@ sub VersionSend {
 }
 
 
-###########################################################################
-#
-# SetXDelegates - Sets the delegates for the <x/> that you might see during
-#                 the session.
-#
-###########################################################################
-sub SetXDelegates {
-  shift;
-  my $self = shift;
-  my $x = new Net::Jabber::X;
-  $x->SetDelegates(@_);
-}
-
-
 1;
