@@ -49,10 +49,11 @@ Net::Jabber::Query::Agent - Jabber Query Agent Module
     $service     = $agent->GetService();
     $register    = $agent->GetRegister();
     $search      = $agent->GetSearch();
+    $groupchat   = $agent->GetGroupChat();
+    $agents      = $agent->GetAgents();
 
 =head2 Creation functions
 
-
     $agent->SetAgent(jid=>"users.jabber.org",
 		     name=>"Jabber User Directory",
 	             description=>"You may register and create a public 
@@ -69,6 +70,8 @@ Net::Jabber::Query::Agent - Jabber Query Agent Module
     $agent->SetService("icq");
     $agent->SetRegister();
     $agent->SetSearch();
+    $agent->SetGroupChat();
+    $agent->SetAgents();
 
 =head1 METHODS
 
@@ -91,6 +94,10 @@ Net::Jabber::Query::Agent - Jabber Query Agent Module
 
   GetSearch() - returns a 1 if the agent supports searching, 0 if not.
 
+  GetGroupChat() - returns a 1 if the agent supports groupchat, 0 if not.
+
+  GetAgents() - returns a 1 if the agent supports sub-agents, 0 if not.
+
 
 =head2 Creation functions
 
@@ -100,8 +107,9 @@ Net::Jabber::Query::Agent - Jabber Query Agent Module
            transport=>string,     attribute twice, the second setting is
            service=>string,       what is used.  If you set the name, and
            register=>string,      then set the search then both will be in
-           search=>string)        the <iq/> tag.  For valid settings read the
-                                  specific Set functions below.
+           search=>string,        the <iq/> tag.  For valid settings read the
+           groupchat=>string)     specific Set functions below.
+           agents=>string)
 
   SetJID(string) - sets the jid="..." of the agent.
 
@@ -121,6 +129,14 @@ Net::Jabber::Query::Agent - Jabber Query Agent Module
                 is put in the <query/> to signify searching is
                 available.
 
+  SetGroupChat() - if the function is called then a <groupchat/> is
+                   is put in the <query/> to signify groupchat is
+                   available.
+
+  SetAgents() - if the function is called then a <agents/> is
+                is put in the <query/> to signify sub-agents are
+                available.
+
 =head1 AUTHOR
 
 By Ryan Eatmon in May of 2000 for http://jabber.org..
@@ -137,7 +153,7 @@ use strict;
 use Carp;
 use vars qw($VERSION);
 
-$VERSION = "1.0";
+$VERSION = "1.0005";
 
 sub new {
   my $proto = shift;
@@ -165,7 +181,7 @@ sub new {
 sub GetJID {
   my $self = shift;
   $self = shift if !exists($self->{AGENT});
-  return &Net::Jabber::GetXMLData("value",(!exists($self->{AGENT}) ? $self->{QUERY} : $self->{AGENT}),"jid","");
+  return &Net::Jabber::GetXMLData("value",(!exists($self->{AGENT}) ? $self->{QUERY} : $self->{AGENT}),"","jid");
 }
 
 
@@ -241,6 +257,30 @@ sub GetSearch {
 }
 
 
+##############################################################################
+#
+# GetGroupChat - returns the namr of the jabber:iq:agent
+#
+##############################################################################
+sub GetGroupChat {
+  my $self = shift;
+  $self = shift if !exists($self->{AGENT});
+  return &Net::Jabber::GetXMLData("existence",(!exists($self->{AGENT}) ? $self->{QUERY} : $self->{AGENT}),"groupchat","");
+}
+
+
+##############################################################################
+#
+# GetAgents - returns the namr of the jabber:iq:agent
+#
+##############################################################################
+sub GetAgents {
+  my $self = shift;
+  $self = shift if !exists($self->{AGENT});
+  return &Net::Jabber::GetXMLData("existence",(!exists($self->{AGENT}) ? $self->{QUERY} : $self->{AGENT}),"agents","");
+}
+
+
 ##############################################################################
 #
 # GetXML - returns the XML string that represents the data in the XML::Parser
@@ -249,6 +289,7 @@ sub GetSearch {
 ##############################################################################
 sub GetXML {
   my $self = shift;
+  $self = shift if !exists($self->{AGENT});
   return &Net::Jabber::BuildXML(@{$self->{AGENT}});
 }
 
@@ -260,7 +301,8 @@ sub GetXML {
 #
 ##############################################################################
 sub GetTree {
-  my $self = shift;  
+  my $self = shift;
+  $self = shift if !exists($self->{AGENT});
   return @{$self->{AGENT}};
 }
 
@@ -283,7 +325,9 @@ sub SetAgent {
   $self->SetTransport($agent{transport}) if exists($agent{transport});
   $self->SetService($agent{service}) if exists($agent{service});
   $self->SetRegister() if exists($agent{register});
-  $self->SetService() if exists($agent{service});
+  $self->SetSearch() if exists($agent{search});
+  $self->SetGroupChat() if exists($agent{groupchat});
+  $self->SetAgents() if exists($agent{agents});
 }
 
 
@@ -296,7 +340,7 @@ sub SetJID {
   my $self = shift;
   $self = shift if !exists($self->{AGENT});
   my ($jid) = @_;
-  &Net::Jabber::SetXMLData("single",(!exists($self->{AGENT}) ? $self->{QUERY} : $self->{AGENT}),"jid",$jid,{});
+  &Net::Jabber::SetXMLData("single",(!exists($self->{AGENT}) ? $self->{QUERY} : $self->{AGENT}),"","",{jid=>$jid});
 }
 
 
@@ -377,6 +421,30 @@ sub SetSearch {
 }
 
 
+##############################################################################
+#
+# SetGroupChat - sets the groupchat in the jabber:iq:agent
+#
+##############################################################################
+sub SetGroupChat {
+  my $self = shift;
+  $self = shift if !exists($self->{AGENT});
+  &Net::Jabber::SetXMLData("single",(!exists($self->{AGENT}) ? $self->{QUERY} : $self->{AGENT}),"groupchat","",{});
+}
+
+
+##############################################################################
+#
+# SetAgents - sets the agents in the jabber:iq:agent
+#
+##############################################################################
+sub SetAgents {
+  my $self = shift;
+  $self = shift if !exists($self->{AGENT});
+  &Net::Jabber::SetXMLData("single",(!exists($self->{AGENT}) ? $self->{QUERY} : $self->{AGENT}),"agents","",{});
+}
+
+
 ##############################################################################
 #
 # debug - prints out the XML::Parser Tree in a readable format for debugging
@@ -384,6 +452,7 @@ sub SetSearch {
 ##############################################################################
 sub debug {
   my $self = shift;
+  $self = shift if !exists($self->{AGENT});
 
   print "debug AGENT: $self\n";
   &Net::Jabber::printData("debug: \$self->{AGENT}->",$self->{AGENT});
