@@ -12,7 +12,7 @@ Net::Jabber::Query::Roster::Item - Jabber IQ Roster Item Module
 
 =head1 DESCRIPTION
 
-  To initialize the Item with a Jabber <iq/> and then access the auth
+  To initialize the Item with a Jabber <iq/> and then access the roster
   query you must pass it the XML::Parser Tree array from the 
   Net::Jabber::Client module.  In the callback function for the iq:
 
@@ -122,8 +122,8 @@ Net::Jabber::Query::Roster::Item - Jabber IQ Roster Item Module
           subscription=>string,   and overwriting action.  If you
           ask=>string,            set the "ask" twice, the second
           groups=>array)          setting is what is used.  If you set
-                                  the password, and then set the
-                                  resource then both will be in the
+                                  the name, and then set the
+                                  jid then both will be in the
                                   <item/> tag.  For valid settings
                                   read the specific Set functions below.
                                   Note: group does not behave in this
@@ -135,13 +135,11 @@ Net::Jabber::Query::Roster::Item - Jabber IQ Roster Item Module
                    Identifiers or the server will return an error message.
                    (ie.  jabber:bob@jabber.org/Silent Bob, etc...)
 
-  SetName(string) - sets the password for the account you are
-                    trying to connect with.  Leave blank for
-                    an anonymous account.
+  SetName(string) - sets the name this roster item should show in the
+                    roster.
 
-  SetSubscription(string) - sets the resource for the account you are
-                            trying to connect with.  Leave blank for
-                            an anonymous account.
+  SetSubscription(string) - sets the subscription that this roster item
+                            has.
 
   SetAsk(string) - sets the ask for the <item/>.
 
@@ -164,7 +162,7 @@ use strict;
 use Carp;
 use vars qw($VERSION);
 
-$VERSION = "1.0";
+$VERSION = "1.0005";
 
 sub new {
   my $proto = shift;
@@ -244,7 +242,12 @@ sub GetAsk {
 sub GetGroups {
   my $self = shift;
 
-  return &Net::Jabber::GetXMLData("value array",$self->{ITEM},"group");
+  my @groups = &Net::Jabber::GetXMLData("value array",$self->{ITEM},"group");
+  my $index;
+  foreach $index (0..$#groups) {
+    splice(@groups,$index,1) if ($groups[$index] eq "");
+  }
+  return @groups;
 }
 
 
@@ -349,10 +352,10 @@ sub SetAsk {
 ##############################################################################
 sub SetGroups {
   my $self = shift;
-  my (@groups) = @_;
+  my ($groups) = @_;
   my ($group);
 
-  foreach $group (@groups) {
+  foreach $group (@{$groups}) {
     &Net::Jabber::SetXMLData("multiple",$self->{ITEM},"group",$group,{});
   }
 }
