@@ -104,10 +104,10 @@ it under the same terms as Perl itself.
 
 require 5.003;
 use strict;
-use Carp;
+use POSIX;
 use vars qw($VERSION);
 
-$VERSION = "1.0";
+$VERSION = "1.0005";
 
 sub new {
   my $proto = shift;
@@ -136,13 +136,13 @@ sub GetName {
 
 ##############################################################################
 #
-# GetVer - returns the timezone in the <query/>.
+# GetVer - returns the version in the <query/>.
 #
 ##############################################################################
 sub GetVer {
   shift;
   my $self = shift;
-  return &Net::Jabber::GetXMLData("value",$self->{QUERY},"ver");
+  return &Net::Jabber::GetXMLData("value",$self->{QUERY},"version");
 }
 
 
@@ -172,7 +172,7 @@ sub SetVersion {
   
   $self->SetName($version{name}) if exists($version{name});
   $self->SetVer($version{ver}) if exists($version{ver});
-  $self->SetOS($version{os}) if exists($version{os});
+  $self->SetOS($version{os});
 }
 
 
@@ -197,8 +197,8 @@ sub SetName {
 sub SetVer {
   shift;
   my $self = shift;
-  my ($ver) = @_;
-  &Net::Jabber::SetXMLData("single",$self->{QUERY},"ver",$ver,{});
+  my ($version) = @_;
+  &Net::Jabber::SetXMLData("single",$self->{QUERY},"version",$version,{});
 }
 
 
@@ -211,6 +211,10 @@ sub SetOS {
   shift;
   my $self = shift;
   my ($os) = @_;
+  if ($os eq "") {
+    my @uname = &POSIX::uname();
+    $os = $uname[0];
+  }
   &Net::Jabber::SetXMLData("single",$self->{QUERY},"os",$os,{});
 }
 
